==> japhar-0.06.tar.gz: japhar-0.06/progs/japharh/japharh.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   japharh.c -- output headers and stubs for java class files.
   Created: Chris Toshok <toshok@hungry.com>
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "classpath.h"
#include "resolve.h"
#include "log.h"
#include "japharh.h"
#include "getopt.h"
#include "util.h"

#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <assert.h>

#define MYLOG "Javah"

char *javah_dest_dir = NULL;
int do_stubs = 0;
static int first_class_index = 1;
char *classpath = NULL;

extern void output_header(ClazzFile*, char*);
extern void output_stubs(ClazzFile*, char*);

void usage(char *name)
{
  printf ("Usage: %s [OPTION]... CLASS [CLASSES]...\n", name);
  printf ("output headers or stubs for each class listed on the command line\n"
	  "  --d=PATH          sets the destination directory to PATH.\n"
	  "  --classpath=PATH  use PATH to lookup classes.\n"
          "  --help            display this help and exit.\n"
          "  --version         output version information and exit.\n"
	  "  --stubs           output stubs for the classes listed.\n"
	  "  --header          output headers for the classes listed (default).\n");
}

static void
parse_command_line(int argc, char **argv)
{
  while (1)
    {
      int c;
      static int version_flag = 0;
      static int help_flag = 0;
      static struct option long_options[] = 
      {
	{ "version", no_argument, &version_flag, 1 },
	{ "help", no_argument, &help_flag, 1 },
	{ "d", required_argument, 0, 'q' },
	{ "classpath", required_argument, 0, 'p' },
	{ "stubs", no_argument, &do_stubs, 1 },
	{ "header", no_argument, &do_stubs, 0 },
	{ 0, 0, 0, 0 }
      };

      int option_index = 0;

      c = getopt_long (argc, argv, "+D:", /* the `+' sets POSIX semantics */
		       long_options, &option_index);

      /* Detect the end of the options. */
      if (c == -1)
        break;

      switch (c)
	{
	case 'p':
	  classpath = joptarg;
	  break;
	case 'q':
	  javah_dest_dir = joptarg;
	  break;
	}

      if (version_flag)
	{
          printf ("GNU Japharh %s\n"
                  "Copyright (C) 1998 The Hungry Programmers\n"
                  "Japhar comes with ABSOLUTELY NO WARRANTY.\n"
                  "You may redistribute copies of japharh\n"
                  "under the terms of the GNU General Public License.\n"
                  "For more information about these matters, see the "
                  "files named COPYING.\n", VERSION);
	  exit(0);
	}

      if (help_flag)
	{
	  usage (argv[0]);
	  exit(0);
	}
    }

  first_class_index = joptind;
}

void
check_dest_dir(char *dest_dir)
{
  struct stat buf;
  int stat_result;

  stat_result = stat(dest_dir, &buf);

  if (stat_result != -1
      && !S_ISDIR(buf.st_mode))
    {
      fprintf (stderr, "Destination directory %s is not a directory\n", dest_dir);
      exit(1);
    }
  else if (stat_result == -1)
    {
      if (mkdir(dest_dir, 0777) == -1)
	{
	  fprintf (stderr, "Could not create the destination directory %s\n", dest_dir);
	  exit(1);
	}
    }
}

int main (int argc, char *argv[])
{
  ClazzFile *cf;
  ClasspathEntry *entries;
  int num_cp_entries;
  int i;
  JNIEnv *env = NULL;
  int free_classpath = 0;

  parse_command_line(argc, argv);

  if (first_class_index == argc)
    {
      usage(argv[0]);
      exit(0);
    }

  /*
   * If no classpath was given on command line, use env CLASSPATH and
   * append default system path.
   */
  if (classpath == NULL)
    {
      classpath = getenv("CLASSPATH");
      if (classpath == NULL)
	classpath = CLASSPATH_getSystemClasspath();
      else
	{
	  char *sys_classpath = CLASSPATH_getSystemClasspath();
	  /* path + : + \0 */
	  int len = strlen(classpath) + strlen(sys_classpath) + 2;
	  char *new_classpath = malloc(sizeof(char)*len);

	  assert(NULL != new_classpath);
	  strcpy(new_classpath, classpath);
	  strcat(new_classpath, ":");
	  strcat(new_classpath, sys_classpath);
	  classpath = new_classpath;
	  free_classpath = 1;
	}
    }
  CLASSPATH_create(classpath, &entries, &num_cp_entries);

  if (free_classpath)
    free(classpath);

  if (javah_dest_dir == NULL)
    javah_dest_dir = "./";

  check_dest_dir(javah_dest_dir);

  for (i = first_class_index; i < argc; i ++)
    {
      char *class_name = argv[i];
      char *slash_class_name = strdup(class_name);

      dots_to_slashes(slash_class_name);

      JAVARLOG1(MYLOG, 1, "  Processing class '%s'...\n", class_name);

      cf = find_class_on_classpath(env, slash_class_name, entries, num_cp_entries);
      
      if (cf == NULL || NULL != strchr(class_name, '/'))
	{
	  fprintf (stderr, "Unable to load class %s\n", class_name);
	  exit(1);
	}
      
      dots_to_underscores(class_name);
      
      if (do_stubs)
        output_stubs(cf, class_name);
      else
        output_header(cf, class_name);

      free(slash_class_name);
    }

  return 0;
}

==> japhar-0.07.tar.gz: japhar-0.07/progs/japharh/japharh.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   japharh.c -- output headers and stubs for java class files.
   Created: Chris Toshok <toshok@hungry.com>
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"
#include "classpath.h"
#include "resolve.h"
#include "log.h"
#include "japharh.h"
#include "getopt.h"
#include "util.h"

#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <assert.h>

#define MYLOG "Javah"

char *javah_dest_dir = NULL;
int do_stubs = 0;
static int first_class_index = 1;
char *classpath = NULL;

extern void output_header(ClazzFile*, char*);
extern void output_stubs(ClazzFile*, char*);

static void usage(char *name)
{
  printf ("Usage: %s [OPTION]... CLASS [CLASSES]...\n", name);
  printf ("output headers or stubs for each class listed on the command line\n"
	  "  --d=PATH          sets the destination directory to PATH.\n"
	  "  --classpath=PATH  use PATH to lookup classes.\n"
          "  --help            display this help and exit.\n"
          "  --version         output version information and exit.\n"
	  "  --stubs           output stubs for the classes listed.\n"
	  "  --header          output headers for the classes listed (default).\n");
}

static void
parse_command_line(int argc, char **argv)
{
  while (1)
    {
      int c;
      static int version_flag = 0;
      static int help_flag = 0;
      static struct option long_options[] = 
      {
	{ "version", no_argument, &version_flag, 1 },
	{ "help", no_argument, &help_flag, 1 },
	{ "d", required_argument, 0, 'q' },
	{ "classpath", required_argument, 0, 'p' },
	{ "stubs", no_argument, &do_stubs, 1 },
	{ "header", no_argument, &do_stubs, 0 },
	{ 0, 0, 0, 0 }
      };

      int option_index = 0;

      c = getopt_long (argc, argv, "+D:", /* the `+' sets POSIX semantics */
		       long_options, &option_index);

      /* Detect the end of the options. */
      if (c == -1)
        break;

      switch (c)
	{
	case 'p':
	  classpath = joptarg;
	  break;
	case 'q':
	  javah_dest_dir = joptarg;
	  break;
	}

      if (version_flag)
	{
          printf ("GNU Japharh %s\n"
                  "Copyright (C) 1998 The Hungry Programmers\n"
                  "Japhar comes with ABSOLUTELY NO WARRANTY.\n"
                  "You may redistribute copies of japharh\n"
                  "under the terms of the GNU General Public License.\n"
                  "For more information about these matters, see the "
                  "files named COPYING.\n", VERSION);
	  exit(0);
	}

      if (help_flag)
	{
	  usage (argv[0]);
	  exit(0);
	}
    }

  first_class_index = joptind;
}

static void
check_dest_dir(char *dest_dir)
{
  struct stat buf;
  int stat_result;

  stat_result = stat(dest_dir, &buf);

  if (stat_result != -1
      && !S_ISDIR(buf.st_mode))
    {
      fprintf (stderr, "Destination directory %s is not a directory\n", dest_dir);
      exit(1);
    }
  else if (stat_result == -1)
    {
      if (mkdir(dest_dir, 0777) == -1)
	{
	  fprintf (stderr, "Could not create the destination directory %s\n", dest_dir);
	  exit(1);
	}
    }
}

int main (int argc, char *argv[])
{
  ClazzFile *cf;
  ClasspathEntry *entries;
  int num_cp_entries;
  int i;
  JNIEnv *env = NULL;
  int free_classpath = 0;

  parse_command_line(argc, argv);

  if (first_class_index == argc)
    {
      usage(argv[0]);
      exit(0);
    }

  /*
   * If no classpath was given on command line, use env CLASSPATH and
   * append default system path.
   */
  if (classpath == NULL)
    {
      classpath = getenv("CLASSPATH");
      if (classpath == NULL)
	classpath = CLASSPATH_getSystemClasspath();
      else
	{
	  char *sys_classpath = CLASSPATH_getSystemClasspath();
	  /* path + : + \0 */
	  int len = strlen(classpath) + strlen(sys_classpath) + 2;
	  char *new_classpath = malloc(sizeof(char)*len);

	  assert(NULL != new_classpath);
	  strcpy(new_classpath, classpath);
	  strcat(new_classpath, ":");
	  strcat(new_classpath, sys_classpath);
	  classpath = new_classpath;
	  free_classpath = 1;
	}
    }
  CLASSPATH_create(classpath, &entries, &num_cp_entries);

  if (free_classpath)
    free(classpath);

  if (javah_dest_dir == NULL)
    javah_dest_dir = "./";

  check_dest_dir(javah_dest_dir);

  for (i = first_class_index; i < argc; i ++)
    {
      char *class_name = argv[i];
      char *slash_class_name = strdup(class_name);

      dots_to_slashes(slash_class_name);

      JAVARLOG1(MYLOG, 1, "  Processing class '%s'...\n", class_name);

      cf = find_class_on_classpath(env, slash_class_name, entries, num_cp_entries);
      
      if (cf == NULL || NULL != strchr(class_name, '/'))
	{
	  fprintf (stderr, "Unable to load class %s\n", class_name);
	  exit(1);
	}
      
      dots_to_underscores(class_name);
      
      if (do_stubs)
        output_stubs(cf, class_name);
      else
        output_header(cf, class_name);

      free(slash_class_name);
    }

  return 0;
}
